@@ -78,8 +78,11 @@ def remove_profile(name):
 @click.group()
 @click.option('-v', '--verbose', count=True, help='Specify log verbosity.')
 @click.option('-n', '--profile', default=DEFAULT_PROFILE)
+@click.option(
+    '-k', '--insecure', is_flag=True, help="Allow Insecure Connection"
+)
 @click.pass_context
-def cli(ctx, verbose, profile):
+def cli(ctx, verbose, profile, insecure):
     """
     Command line wrapper for the Appknox API
     """
@@ -90,6 +93,8 @@ def cli(ctx, verbose, profile):
     else:
         ctx.obj['LOG_LEVEL'] = logging.WARNING
 
+    ctx.obj['INSECURE'] = insecure
+
     ctx.obj['PROFILE'] = profile
 
     ctx.obj['USING_ENV_TOKEN'] = False
@@ -130,7 +135,10 @@ def cli(ctx, verbose, profile):
         username=profile.get('username'), host=profile.get('host'),
         token=profile.get('token'), access_token=profile.get('access_token'),
         user_id=profile.get('user_id'),
-        organization_id=profile.get('organization_id')
+        organization_id=profile.get('organization_id'),
+        http_proxy=os.environ.get('HTTP_PROXY'),
+        https_proxy=os.environ.get('HTTPS_PROXY'),
+        insecure=ctx.obj['INSECURE']
     )
 
 
@@ -145,7 +153,11 @@ def login(ctx, username, password, host):
     """
     ctx.obj['CLIENT'] = client = Appknox(
         username=username, password=password, host=host,
-        log_level=ctx.obj['LOG_LEVEL'])
+        log_level=ctx.obj['LOG_LEVEL'],
+        http_proxy=os.environ.get('HTTP_PROXY'),
+        https_proxy=os.environ.get('HTTPS_PROXY'),
+        insecure=ctx.obj['INSECURE'],
+        )
     try:
         client.login()
     except requests.exceptions.InvalidSchema as e:
@@ -341,17 +353,6 @@ def owasp(ctx, owasp_id):
 #         sys.exit(1)
 
 
-@cli.command()
-@click.argument('file_id')
-@click.pass_context
-def rescan(ctx, file_id):
-    """
-    Start rescanning a file
-    """
-    client = ctx.obj['CLIENT']
-    client.rescan(file_id)
-
-
 @cli.command('switch_organization')
 @click.argument('organization_id')
 @click.pass_context
