@@ -10,7 +10,8 @@ from urllib.parse import urljoin
 
 from appknox.exceptions import (
     OneTimePasswordError, CredentialError, AppknoxError,
-    OrganizationError, UploadError
+    OrganizationError, UploadError, SubmissionNotFound,
+    SubmissionError, SubmissionFileTimeoutError, RescanError
 )
 from appknox.mapper import (
     mapper_json_api, mapper_drf_api, Analysis, File, Project, User,
@@ -36,7 +37,8 @@ class Appknox(object):
         self, username: str = None, password: str = None, user_id: int = None,
         organization_id: int = None, token: str = None,
         access_token: str = None, host: str = DEFAULT_API_HOST,
-        log_level: int = logging.INFO
+        log_level: int = logging.INFO, http_proxy: str = None,
+        https_proxy: str = None, insecure: bool = False,
     ):
         """
         Initialise Appknox client
@@ -60,6 +62,18 @@ class Appknox(object):
         self.organization_id = organization_id
         self.token = token
         self.access_token = access_token
+        self.request_session = requests.session()
+        self.request_session.verify = not insecure  # if insecure don't verify
+        self.proxies = {}
+
+        if http_proxy:
+            self.proxies['http'] = http_proxy
+
+        if https_proxy:
+            self.proxies['https'] = https_proxy
+
+        if self.proxies:
+            self.request_session.proxies.update(self.proxies)
 
         if self.host is None:
             self.host = DEFAULT_API_HOST
@@ -69,25 +83,29 @@ class Appknox(object):
                 'Authorization': 'Token {}'.format(self.access_token)
             }
             self.json_api = ApiResource(
+                self.request_session,
                 host=self.host,
-                headers={**JSON_API_HEADERS, **token_header}
+                headers={**JSON_API_HEADERS, **token_header},
             )
             self.drf_api = ApiResource(
+                self.request_session,
                 host=self.host,
-                headers={**DRF_API_HEADERS, **token_header}
+                headers={**DRF_API_HEADERS, **token_header},
             )
             self.organization_id = self.get_organization_id()
 
         elif self.user_id and self.token:
             self.json_api = ApiResource(
+                self.request_session,
                 host=self.host,
                 headers={**JSON_API_HEADERS},
-                auth=(self.user_id, self.token)
+                auth=(self.user_id, self.token),
             )
             self.drf_api = ApiResource(
+                self.request_session,
                 host=self.host,
                 headers={**DRF_API_HEADERS},
-                auth=(self.user_id, self.token)
+                auth=(self.user_id, self.token),
             )
             self.organization_id = self.get_organization_id()
 
@@ -109,7 +127,8 @@ class Appknox(object):
         if otp:
             data['otp'] = str(otp)
 
-        response = requests.post(urljoin(self.host, 'api/login'), data=data)
+        response = self.request_session.post(
+            urljoin(self.host, 'api/login'), data=data)
 
         if response.status_code == 401:
             raise OneTimePasswordError(response.json()['message'])
@@ -124,16 +143,18 @@ class Appknox(object):
         self.access_token = self.generate_access_token().key
 
         self.json_api = ApiResource(
+            self.request_session,
             host=self.host,
             headers={**JSON_API_HEADERS, **{
                 'Authorization': 'Token {}'.format(self.access_token)
-            }}
+            }},
         )
         self.drf_api = ApiResource(
+            self.request_session,
             host=self.host,
             headers={**DRF_API_HEADERS, **{
                 'Authorization': 'Token {}'.format(self.access_token)
-            }}
+            }},
         )
         self.organization_id = self.get_organization_id()
 
@@ -168,14 +189,14 @@ class Appknox(object):
         """
         Generates personal access token
         """
-        access_token = requests.post(
+        access_token = self.request_session.post(
             urljoin(self.host, 'api/personaltokens'),
             auth=(self.user_id, self.token),
             data={
                 'name': 'appknox-python for {} @{}'.format(
                     self.username, str(int(time.time()))
                 )
-            }
+            },
         )
         return mapper_json_api(PersonalToken, access_token.json())
 
@@ -183,9 +204,9 @@ class Appknox(object):
         """
         Revokes existing personal access token
         """
-        resp = requests.get(
+        resp = self.request_session.get(
             urljoin(self.host, 'api/personaltokens?key=' + self.access_token),
-            auth=(self.user_id, self.token)
+            auth=(self.user_id, self.token),
         )
         resp_json = resp.json()
         personal_token = next((p for p in resp_json.get('data')), None)
@@ -193,9 +214,9 @@ class Appknox(object):
             return
 
         token_id = personal_token['id']
-        return requests.delete(
+        return self.request_session.delete(
             urljoin(self.host, 'api/personaltokens/' + token_id),
-            auth=(self.user_id, self.token)
+            auth=(self.user_id, self.token),
         )
 
     def get_user(self, user_id: int) -> User:
@@ -410,7 +431,7 @@ class Appknox(object):
         pcidss = self.drf_api['v2/pcidsses'](pcidss_id).get()
         return mapper_drf_api(PCIDSS, pcidss)
 
-    def upload_file(self, file_data: str):
+    def upload_file(self, file_data: str) -> int:
         """
         Upload and scan a package and returns the file_id
 
@@ -420,7 +441,7 @@ class Appknox(object):
             'organizations/{}/upload_app'.format(self.organization_id)
         ]().get()
         url = response['url']
-        requests.put(url, data=file_data)
+        self.request_session.put(url, data=file_data)
         response2 = self.drf_api[
             'organizations/{}/upload_app'.format(self.organization_id)
         ]().post(
@@ -430,21 +451,36 @@ class Appknox(object):
                 url=response['url']
             )
         )
+        submission_id = response2['submission_id']
+        try:
+            file_id = self.poll_for_file_from_submission_id(submission_id)
+        except (SubmissionNotFound, SubmissionError):
+            raise UploadError('Something went wrong, try uploading the\
+             file again.')
+        return file_id
+
+    def poll_for_file_from_submission_id(self, submission_id: int) -> int:
+        """
+        Using the submission id, keep checking its status.
+        Returns file instance when it's available
+
+        :param submission_id: The ID of the submission object
+         created for the scan
+
+         :return: The File ID
+        """
         file = None
         timeout = time.time() + 10
-        submission_id = response2['submission_id']
         while (file is None):
             submission = self.drf_api.submissions(submission_id).get()
             if submission.get('detail') == 'Not found.':
-                raise UploadError(
-                    'Something went wrong, try uploading the file again.')
+                raise SubmissionNotFound()
             submission_obj = mapper_drf_api(Submission, submission)
             file = submission_obj.file
             if submission_obj.reason:
-                raise UploadError(submission_obj.reason)
+                raise SubmissionError(submission_obj.reason)
             if time.time() > timeout:
-                raise RuntimeError(
-                    'Request timed out, try uploading the file again.')
+                raise SubmissionFileTimeoutError()
         return file
 
     def recent_uploads(self) -> List[Submission]:
@@ -455,17 +491,25 @@ class Appknox(object):
 
         return self.paginated_drf_data(submissions, Submission)[0:10]
 
-    def rescan(self, file_id: int):
+    def rescan(self, file_id: int) -> int:
         """
         Start a rescan for a file id
 
         :param filed_id: File ID
+
+        :return: The ID of the File created in rescan
         """
-        self.drf_api['rescan']().post(
-            data=dict(
-                file_id=file_id,
-            )
+        endpoint = 'v2/files/{}/rescan'.format(file_id)
+        response = self.drf_api[endpoint]().post(
+            data=dict()
+
         )
+        submission_id = response['submission_id']
+        try:
+            file = self.poll_for_file_from_submission_id(submission_id)
+        except (SubmissionNotFound, SubmissionError):
+            raise RescanError('Something went wrong, retry rescan')
+        return file
 
     # def get_report(
     #         self, file_id, format: str = 'json', language: str = 'en') ->
@@ -493,12 +537,13 @@ class Appknox(object):
 
 class ApiResource(object):
     def __init__(
-        self, host: str = DEFAULT_API_HOST, headers: object = None,
-        auth: Dict[str, str] = None
+        self, request_session: object, host: str = DEFAULT_API_HOST,
+        headers: object = None, auth: Dict[str, str] = None
     ):
         self.host = host
         self.headers = {**headers}
         self.auth = auth
+        self.request_session = request_session
 
         self.endpoint = urljoin(host, API_BASE)
 
@@ -515,21 +560,20 @@ class ApiResource(object):
         return self
 
     def get(self, **kwargs):
-        resp = requests.get(
-            self.endpoint, headers=self.headers, auth=self.auth,
-            params=kwargs
+        resp = self.request_session.get(
+            self.endpoint, headers=self.headers, auth=self.auth, params=kwargs,
         )
         return resp.json()
 
     def post(self, data, content_type=None, **kwargs):
-        resp = requests.post(
+        resp = self.request_session.post(
             self.endpoint, headers=self.headers, auth=self.auth,
-            params=kwargs, data=data
+            params=kwargs, data=data,
         )
         return resp.json()
 
     def direct_get(self, url, **kwargs):
-        resp = requests.get(
-            url, headers=self.headers, auth=self.auth, params=kwargs
+        resp = self.request_session.get(
+            url, headers=self.headers, auth=self.auth, params=kwargs,
         )
         return resp.json()
